import matplotlib.pyplot as plt
from sklearn import datasets
from sklearn.cluster import KMeans
import pandas as pd
import numpy as np
from PIL import Image as Image1
from IPython.display import Image as Image2,display
def problem_statement():
    print('''
Problem Statement: 
    Apply EM algorithm to cluster a set of data stored in a .CSV file. Use the same data set for clustering using k-Means algorithm.Compare the results of these two algorithms and comment on the quality of clustering. You can add Java/Python ML library classes/API in the program.''')
def description():
    print('Description:\n\tEM algorithm is a widely used approach to learning in the presence of unobserved variables. The EM algorithm can be used even for variables whose value is never directly observed, provided the general form of the probability distribution governing these variables is known. The EM algorithm has been used to train Bayesian belief networks as well as radial basis function networks. The EM algorithm is also the basis for many unsupervised clustering algorithms and it is the basis for the widely used Baum-Welch forward-backward algorithm for learning Partially Observable Markov Models.')
def algorithm():
    try:
        display(Image2('em_and_k_means.png'))
    except:
        Image1.open('em_and_k_means.png').show()
def code():
    print('Program for comparing EM and KMeans Clustering algorithms: ')
    print('''
import matplotlib.pyplot as plt
from sklearn import datasets
from sklearn.cluster import KMeans
import pandas as pd
import numpy as np

iris = datasets.load_iris()
X = pd.DataFrame(iris.data)
X.columns = ['Sepal_Length','Sepal_Width','Petal_Length','Petal_Width']
y = pd.DataFrame(iris.target)
y.columns = ['Targets']

model = KMeans(n_clusters=3)
model.fit(X)

plt.figure(figsize=(14,14))
colorMap = np.array(['red','lime','black'])
plt.subplot(2,2,1)
plt.scatter(X.Petal_Length , X.Petal_Width , c = colorMap[y.Targets],s = 40)
plt.title('Real Clusters')
plt.xlabel('Petal Length')
plt.ylabel('Petal Width')

plt.subplot(2,2,2)
plt.scatter(X.Petal_Length , X.Petal_Width , c = colorMap[model.labels_],s = 40)
plt.title('K-Means Clustering')
plt.xlabel('Petal Length')
plt.ylabel('Petal Width')

from sklearn import preprocessing as pp
scaler = pp.StandardScaler()
scaler.fit(X)
xsa = scaler.transform(X)
xs = pd.DataFrame(xsa , columns=X.columns)

from sklearn.mixture import GaussianMixture as gm
gmm = gm(n_components=3)
gmm.fit(xs)
gmm_y = gmm.predict(xs)
plt.subplot(2,2,3)
plt.scatter(X.Petal_Length,X.Petal_Width,c= colorMap[gmm_y],s=40)
plt.title('GM Clustering')
plt.xlabel('Petal Length')
plt.ylabel('Petal Width')
plt.show()
print(\'''
Observation:
    The GMM using EM based Algorithm clustering matched the labels more closely than the KMeans.
    \''')
    ''')
def run():
    iris = datasets.load_iris()
    X = pd.DataFrame(iris.data)
    X.columns = ['Sepal_Length', 'Sepal_Width', 'Petal_Length', 'Petal_Width']
    y = pd.DataFrame(iris.target)
    y.columns = ['Targets']

    model = KMeans(n_clusters=3)
    model.fit(X)

    plt.figure(figsize=(14, 14))
    colorMap = np.array(['red', 'lime', 'black'])
    plt.subplot(2, 2, 1)
    plt.scatter(X.Petal_Length, X.Petal_Width, c=colorMap[y.Targets], s=40)
    plt.title('Real Clusters')
    plt.xlabel('Petal Length')
    plt.ylabel('Petal Width')

    plt.subplot(2, 2, 2)
    plt.scatter(X.Petal_Length, X.Petal_Width, c=colorMap[model.labels_], s=40)
    plt.title('K-Means Clustering')
    plt.xlabel('Petal Length')
    plt.ylabel('Petal Width')

    from sklearn import preprocessing as pp
    scaler = pp.StandardScaler()
    scaler.fit(X)
    xsa = scaler.transform(X)
    xs = pd.DataFrame(xsa, columns=X.columns)

    from sklearn.mixture import GaussianMixture as gm
    gmm = gm(n_components=3)
    gmm.fit(xs)
    gmm_y = gmm.predict(xs)
    plt.subplot(2, 2, 3)
    plt.scatter(X.Petal_Length, X.Petal_Width, c=colorMap[gmm_y], s=40)
    plt.title('GM Clustering')
    plt.xlabel('Petal Length')
    plt.ylabel('Petal Width')
    plt.show()
    print('''
Observation:
    The GMM using EM based Algorithm clustering matched the labels more closely than the KMeans.
        ''')


# if __name__ == '__main__':
#     run()


import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from PIL import Image as Image1
from IPython.display import Image as Image2,display
def kernel(point, xmat, k , x):
    m, n = np.shape(xmat)
    weights = np.mat(np.eye((m)))
    for j in range(m):
        diff = point - x[j]
        weights[j, j] = np.exp(diff * diff.T / (-2.0 * k ** 2))
    return weights


def LocalWeight(point, xmat, ymat, k , x):
    wei = kernel(point, xmat, k , x)
    w = (x.T * (wei * x)).I * (x.T * (wei * ymat.T))
    return w


def localWeightRegression(xmat, ymat, k , x):
    m, n = np.shape(xmat)
    ypred = np.zeros(m)
    for i in range(m):
        ypred[i] = xmat[i] * LocalWeight(xmat[i], xmat, ymat, k , x)
    return ypred
def graphPlot(x, ypred,bill,tip):
    sortindex = x[:, 1].argsort(0)
    xsort = x[sortindex][:, 0]
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.scatter(bill, tip, color='green')
    ax.plot(xsort[:, 1], ypred[sortindex], color='red', linewidth=5)
    plt.xlabel('Total Bill')
    plt.ylabel('Tip')
    plt.show()

def problem_statement():
    print('''
Problem Statement: 
    Implement the non-parametric Locally Weighted Regression algorithm in order to fit data points. 
    Select appropriate data set for your experiment and draw graphs.''')
def description():
    print('Description:\n\tLocally weighted regression is a generalization of k-nearest neighbour approach. It constructs an explicit approximation to f over a local region surrounding xq. Locally weighted regression uses nearby or distance-weighted training examples to form this local approximation to f. For example, we might approximate the target function in the neighborhood surrounding xq. Using a linear function, a quadratic function, a multilayer neural network, or some other functional form. The phrase “locally weighted regression” is called local because the function is approximated based only on data near the query point, weighted because the contribution of each training example is weighted by its distance from the query point, and regression because this is the term used widely in the statistical learning community for the problem of approximating real-valued functions.')
def algorithm():
    try:
        display(Image2('locally_weighted_regression.png'))
    except:
        Image1.open('locally_weighted_regression.png').show()
def code():
    print('\nProgram for Locally Weighted Regression: ')
    print('''
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
def kernel(point,xmat,k):
    m,n=np.shape(xmat)
    weights=np.mat(np.eye((m)))
    for j in range(m):
        diff=point-x[j]
        weights[j,j]=np.exp(diff*diff.T/(-2.0*k**2))
    return weights

def LocalWeight(point,xmat,ymat,k):
    wei=kernel(point,xmat,k)
    w=(x.T*(wei*x)).I*(x.T*(wei*ymat.T))
    return w

def localWeightRegression(xmat,ymat,k):
    m,n=np.shape(xmat)
    ypred=np.zeros(m)
    for i in range(m):
        ypred[i]=xmat[i]*LocalWeight(xmat[i],xmat,ymat,k)
    return ypred

def graphPlot(x,ypred):
    sortindex=x[:,1].argsort(0)
    xsort=x[sortindex][:,0]
    fig=plt.figure()
    ax=fig.add_subplot(1,1,1)
    ax.scatter(bill,tip,color='green')
    ax.plot(xsort[:,1],ypred[sortindex],color='red',linewidth=5)
    plt.xlabel('Total Bill')
    plt.ylabel('Tip')
    plt.show()
    
data=pd.read_csv('Tips.csv')
bill=np.array(data.total_bill)
tip=np.array(data.tip)
mbill=np.mat(bill)
mtip=np.mat(tip)
m=np.shape(mbill)[1]
one=np.mat(np.ones(m))
x=np.hstack((one.T,mbill.T))
ypred=localWeightRegression(x,mtip,0.5)
graphPlot(x,ypred)
    ''')
def run(path):
    data = pd.read_csv(path)
    bill = np.array(data.total_bill)
    tip = np.array(data.tip)
    mbill = np.mat(bill)
    mtip = np.mat(tip)
    m = np.shape(mbill)[1]
    one = np.mat(np.ones(m))
    x = np.hstack((one.T, mbill.T))
    ypred = localWeightRegression(x, mtip, 0.5 , x)
    graphPlot(x, ypred,bill,tip)



def astar(start,end,tree,heuristic,cost):
    opened, closed = [[start, 0]], []
    while True:
        fn = [i[1] for i in opened]
        chosen_index = fn.index(min(fn))
        node = opened[chosen_index][0]
        closed.append(opened[chosen_index])
        del opened[chosen_index]
        if closed[-1][0] == end:
            break
        for item in tree[node]:
            if item[0] in [i[0] for i in closed]:
                continue
            cost.update({item[0]: item[1] + cost[node]})
            opened.append([item[0],item[1] + cost[node] + heuristic[item[0]]])
    trace_node, optimal_sequence = 'E', ['E']
    for i in range(len(closed) - 2, -1, -1):
        check_node = closed[i][0]
        if trace_node in [i[0] for i in tree[check_node]]:
            children_cost = [i[1] for i in tree[check_node]]
            children_node = [i[0] for i in tree[check_node]]
            if cost[check_node] + children_cost[children_node.index(trace_node)] == cost[trace_node]:
                optimal_sequence.append(check_node)
                trace_node = check_node
    optimal_sequence.reverse()
    return closed, optimal_sequence

def algorithm():
    print('''\nAlgorithm: The A* search algorithm is discussed below:
    1) Initialize the open and closed lists
    2) Add start node to the open list
    3) For all the neighbouring nodes, find the least cost F node
    4) Switch to the closed list
        * For nodes adjacent to the current node
        * If the node is not reachable, ignore it. 
        * Else If the node is not on the open list, move it to the open list and calculate f, g, h.
        * If the node is on the open list, check if the path it offers is less than the current path 
          and change to it if it does so.
    5) Stop working when
        * You find the destination
        * You cannot find the destination going through all possible points'''
          )
def problem_statement():
    print('''
Problem Statement:
    Implement A* Search Algorithm.''')
def description():
    print('\nDescription: \n\tA* Search algorithm is one of the best and popular technique used in path-finding and graph traversals. Informally speaking, A* Search algorithms, unlike other traversal techniques, it has “brains”. What it means is that it is really a smart algorithm which separates it from the other conventional algorithms. Example: Consider a square grid having many obstacles and we are given a starting cell and a target cell. We want to reach the target cell from the starting cell as quickly as possible. What A* Search Algorithm does is that at each step it picks the node according to a value-‘f’ which is a parameter equal to the sum of two other parameters – ‘g’ and ‘h’. At each step it picks the node/cell having the lowest ‘f’, and processes that node/cell. The two parameters ‘g’ and ‘h’ are defined as follows: g = the movement cost to move from the starting point to a given square on the grid, following the path generated to get there. h = the estimated movement cost to move from that given square on the grid to the final destination.')

def code():
    print('\nProgram for A* Search Algorithm:')
    print('''
tree = {'S': [['A', 1], ['B', 2]],
        'A': [['E', 13]],
        'B': [['E', 5]]}
heuristic = {'S': 5, 'A': 4, 'B': 5, 'E': 0}
cost = {'S': 0}

def astar():
    opened, closed = [['S', 0]], []
    while True:
        fn = [i[1] for i in opened]
        chosen_index = fn.index(min(fn))
        node = opened[chosen_index][0]
        closed.append(opened[chosen_index])
        del opened[chosen_index]
        if closed[-1][0] == 'E':
            break
        for item in tree[node]:
            if item[0] in [i[0] for i in closed]:
                continue
            cost.update({item[0]: item[1] + cost[node]})
            opened.append([item[0], item[1] + cost[node] + heuristic[item[0]]])
    trace_node, optimal_sequence = 'E', ['E']
    for i in range(len(closed) - 2, -1, -1):
        check_node = closed[i][0]
        if trace_node in [i[0] for i in tree[check_node]]:
            children_cost = [i[1] for i in tree[check_node]]
            children_node = [i[0] for i in tree[check_node]]
            if cost[check_node] + children_cost[children_node.index(trace_node)] == cost[trace_node]:
                optimal_sequence.append(check_node)
                trace_node = check_node
    optimal_sequence.reverse()
    return closed, optimal_sequence

if __name__ == '__main__':
    print(astar())

    ''')

# if __name__ == '__main__':
#     run()


class Graph:
    def __init__(self, graph, hnl, startNode):
        self.graph = graph
        self.H = hnl
        self.start = startNode
        self.parent = {}
        self.status = {}
        self.solutionGraph = {}

    def applyAOstar(self):
        self.aoStar(self.start, False)

    def getNeighbors(self, v):
        return self.graph.get(v, '')

    def getStatus(self, v):
        return self.status.get(v, 0)

    def setStatus(self, v, val):
        self.status[v] = val

    def gethnv(self, n):
        return self.H.get(n, 0)

    def sethnv(self, n, val):
        self.H[n] = val

    def printSolution(self):
        print("FOR GRAPH SOLUTION, TRAVERSE THE GRAPH FROM THE START NODE : ", self.start)
        print(
            "---------------------------------------------------------------------------------------------------------------")
        print(self.solutionGraph)
        print(
            "---------------------------------------------------------------------------------------------------------------")
    def cmccn(self, v):
        mc = 0
        ctcn = {}
        ctcn[mc] = []
        flag = True
        for element in self.getNeighbors(v):
            cost = 0
            nl = []
            for c, weight in element:
                cost = cost + self.gethnv(c) + weight
                nl.append(c)
            if flag == True:
                mc = cost
                ctcn[mc] = nl
                flag = False
            else:
                if mc > cost:
                    mc = cost
                    ctcn[mc] = nl
        return mc, ctcn[mc]

    def aoStar(self, v, bt):
        print("HEURISTIC VALUE : ", self.H)
        print("SOLUTION GRAPH : ", self.solutionGraph)
        print("PROCESSING NODE : ", v)
        print(
            "---------------------------------------------------------------------------------------------------------------")
        if self.getStatus(v) >= 0:
            mc, cnl = self.cmccn(v)
            self.sethnv(v, mc)
            self.setStatus(v, len(cnl))
            solved = True
            for cn in cnl:
                self.parent[cn] = v
                if self.getStatus(cn) != -1:
                    solved = solved & False
            if solved:
                self.setStatus(v, -1)
                self.solutionGraph[v] = cnl
            if v != self.start:
                self.aoStar(self.parent[v], True)
            if not bt:
                for cn in cnl:
                    self.setStatus(cn, 0)
                    self.aoStar(cn, False)

#
# def run(graph , heuristicNodeList , startNode):
#     h1 = {'A': 1, 'B': 6, 'C': 2, 'D': 12, 'E': 2, 'F': 1, 'G': 5, 'H': 7, 'I': 0, 'J': 1, 'T': 3}
#     graph1 = {
#         'A': [[('B', 1), ('C', 1)], [('D', 1)]],
#         'B': [[('G', 1)], [('H', 1)]],
#         'C': [[('J', 1)]],
#         'D': [[('E', 1), ('F', 1)]],
#         'G': [[('I', 1)]]
#     }
#     G1 = Graph(graph , heuristicNodeList , startNode)
#     G1.applyAOstar()
#     G1.printSolution()
#
#     h2 = {'A': 1, 'B': 6, 'C': 12, 'D': 10, 'E': 4, 'F': 4, 'G': 5, 'H': 7}
#     graph2 = {
#         'A': [[('B', 1), ('C', 1)], [('D', 1)]],
#         'B': [[('G', 1)], [('H', 1)]],
#         'D': [[('E', 1), ('F', 1)]]
#     }
#     G2 = Graph(graph2, h2, 'A')
#     G2.applyAOstar()
#     G2.printSolution()

def problem_statement():
    print('''
Problem Statement:
    Implement AO* Search algorithm
    ''')
def description():
    print('Description:\n\tAO* Algorithm basically based on  problem decomposition (Breakdown problem into small pieces).When a problem can be divided into a set of sub problems, where each sub problem can be solved separately and a combination of these will be a solution, AND-OR graphs or AND - OR trees are used for representing the solution. The decomposition of the problem or problem reduction generates AND arcs.')
def algorithm():
    print('''
Algorithm: The AO* search algorithm is discussed below:
    1. Let GRAPH consist only of the node representing the initial state. Call this node INIT, Compute h' (INIT). 
    2. Until INIT is labelled SOLVED or until INIT’s h' value becomes greater than FUTILITY, repeat the following procedure:
        a. Trace the labelled arcs from INIT and select for expansion one of the as yet unexpanded nodes that occurs on 
           this path. Call the selected node NODE.
        b. Generate the successors of NODE. If there are none, then assign FUTILITY as the h' value of NODE. 
           This is equivalent to saying that NODE is not solvable. If there are successors, then for each one (called SUCCESSOR) do the following:
            i) Add SUCCESSOR to GRAPH.
            ii) If SUCCESSOR is a terminal node, label it SOLVED & assign it an h' value to 0.
            iii) If SUCCESSOR is not a terminal node, compute its h' value.
        c. Propagate the newly discovered information up the graph by doing the following: Let S be a set of nodes that have 
           been labelled SOLVED or whose h' values have been changed and so need to have values propagated back to their parents.
           Initialize S to NODE. Until S is empty, repeat the following procedure:
            i) If possible, select from S a node none of whose descendants in GRAPH occurs in S. If there is no such node, 
               select any node from S. Call this node CURRENT, and remove it from S.
            ii) Compute the cost of each of the arcs emerging from CURRENT. The cost of each arc is equal to the sum of the h' 
                values of each of the nodes at the end of the arc plus whatever the cost of the arc itself is. Assign as 
                CURRENT’S new h' value the minimum of the costs just computed for the arcs emerging from it.
            iii) Mark the best path out of CURRENT by making the arc that had the minimum cost as compared in the previous step.
            iv) Mark CURRENT SOLVED if all the nodes connected to it through the new labelled arc have been labelled SOLVED.
            v) If CURRENT has been labelled SOLVED or if the cost of CURRENT was just changed, then its new status must be 
               propagated back up the graph. 
            So add all of the ancestors of CURRENT to S.
    ''')
def code():
    print('\n Program for AO* algorithm: ')
    print('''
class Graph:
    def __init__(self, graph, heuristicNodeValue, startNode):
        self.graph = graph
        self.H = heuristicNodeValue
        self.start = startNode
        self.parent = {}
        self.status = {}
        self.solutionGraph = {}

    def getNeighbour(self, v):
        return self.graph.get(v, '')

    def getStatus(self, v):
        return self.status.get(v, 0)

    def setStatus(self, v, val):
        self.status[v] = val

    def getH(self, v):
        return self.H.get(v, 0)

    def setH(self, v, val):
        self.H[v] = val

    def printSolution(self):
        print('FOR THE GRAPH SOLUTION , TRAVERSE THE GRAPH FROM THE NODE : ', self.start)
        print('-' * 100)
        print(self.solutionGraph)
        print('-' * 100)

    def applyAOStar(self):
        self.AOStar(self.start, False)

    def minimumCostChild(self, v):
        minimumCost = 0
        ChildNodeList = {minimumCost: []}
        flag = True
        for item in self.getNeighbour(v):
            cost, nodeList = 0, []
            for c, w in item:
                cost += self.getH(c) + w
                nodeList.append(c)
            if flag:
                minimumCost = cost
                ChildNodeList[minimumCost] = nodeList
                flag = False
            else:
                if minimumCost > cost:
                    minimumCost = cost
                    ChildNodeList[minimumCost] = nodeList
        return minimumCost, ChildNodeList[minimumCost]

    def AOStar(self, v, backtracking):
        print(

            'Heuristic Value : ', self.H,
            '\\nSolution Graph : ', self.solutionGraph,
            '\\nProcessing Node : ', v,'\\n',
            '-' * 100
        )
        if self.getStatus(v) >= 0:
            minimumCost, childNodeList = self.minimumCostChild(v)
            self.setH(v, minimumCost)
            self.setStatus(v, len(childNodeList))
            Solved = True
            for childNode in childNodeList:
                self.parent[childNode] = v
                if self.getStatus(childNode) != -1:
                    Solved &= False
            if Solved:
                self.setStatus(v, -1)
                self.solutionGraph[v] = childNodeList
            if v != self.start:
                self.AOStar(self.parent[v], True)
            if not backtracking:
                for childNode in childNodeList:
                    self.setStatus(childNode, 0)
                    self.AOStar(childNode, False)


if __name__ == '__main__':
    h = {'A': 1, 'B': 6, 'C': 12, 'D': 10, 'E': 4, 'F': 4, 'G': 5, 'H': 7}
    graph = {
        'A': [[('B', 1), ('C', 1)], [('D', 1)]],
        'B': [[('G', 1)], [('H', 1)]],
        'D': [[('E', 1), ('F', 1)]]
    }
    g = Graph(graph, h, 'A')
    g.applyAOStar()
    g.printSolution()

    ''')


import math
import csv
from PIL import Image as Image1
from IPython.display import Image as Image2,display
def load_csv(filename):
    lines = csv.reader(open(filename,'r'))
    dataset = list(lines)
    header = dataset.pop(0)
    return dataset , header
class Node:
    def __init__(self,attribute):
        self.attribute = attribute
        self.children = []
        self.answer = ''
def subtables(data , col , delete):
    dic = {}
    coldata = [row[col] for row in data]
    attr = list(set(coldata))
    for k in attr:
        dic[k] = []
    for y in range(len(data)):
        key = data[y][col]
        if delete:
            del data[y][col]
        dic[key].append(data[y])
    return attr , dic

def entropy(S):
    attr = list(set(S))
    if len(attr) == 1:
        return 0
    counts = [0,0]
    for i in range(2):
        counts[i] = sum( [1 for x in S if attr[i] == x]) / (len(S) * 1.0)
    sums  = 0
    for cnt in counts:
        sums += -1 * cnt * math.log(cnt,2)
    return sums

def compute_gain(data , col):
    attrValues , dic = subtables(data ,col , delete= False)
    total_entropy = entropy([row[-1] for row in data])
    for x in range(len(attrValues)):
        ratio = len(dic[attrValues[x]]) / (len(data) * 1.0 )
        entro = entropy([row[-1] for row in dic[attrValues[x]]])
        total_entropy -= ratio*entro
    return total_entropy

def build_tree(data , features):
    lastcol = [row[-1] for row in data]
    if(len(set(lastcol))) == 1:
        node = Node("")
        node.answer = lastcol[0]
        return node
    n  = len(data[0]) -1
    gains = [compute_gain(data , col) for col in range(n)]
    split = gains.index(max(gains))
    node = Node(features[split])
    fea = features[:split] + features[split+1:]
    attr , dic = subtables(data , split , delete=True)
    for x in range(len(attr)):
        child = build_tree(dic[attr[x]],fea)
        node.children.append((attr[x],child))
    return node
def print_tree(node , level):
    if node.answer != "":
        print(' '*level,node.answer)
        return
    print(' '*(level+1),node.attribute)
    for value , n in node.children:
        print(' '*(level+1),value)
        print_tree(n , level + 2)
def classify(node , x_test , features):
    if node.answer != "":
        print(node.answer)
        return
    pos = features.index(node.attribute)
    for value , n in node.children:
        if x_test[pos] == value:
            classify(n ,x_test,features)
def problem_statement():
    print('''
Problem Statement:
    Write a program to demonstrate the working of decision tree based ID3 algorithm. Use an appropriate dataset to build the decision tree & apply 
    this knowledge to test the new sample.
    ''')
def description():
    print('Description:\n\tIterative Dichotomiser (ID3) Heuristic algorithm is used for building the decision tree. It considers the original set of attributes as the root node. On each iteration of the algorithm, we iterate through every unused attribute of the remaining set and calculates the entropy (or information gain) of that attribute. Then, we select the attribute which has the smallest entropy (or largest information gain) value. The set of remaining attributes is then split by the selected attribute to produce subsets of the data. The algorithm continues to recurse on each subset, considering only attributes never selected before. In testing phase at runtime, we will use trained decision tree to classify the new unseen test cases by working down the decision tree using the values of this test case to arrive at a terminal node that tells us what class this test case belongs to.')
def algorithm():
    try:
        display(Image2('id3.jpg'))
    except:
        Image1.open('id3.jpg').show()
def code():
    print('\nProgram for ID3 Algorithm: ')
    print('''
import math
import csv
def load_csv(filename):
    lines = csv.reader(open(filename,'r'))
    dataset = list(lines)
    header = dataset.pop(0)
    return dataset , header
class Node:
    def __init__(self,attribute):
        self.attribute = attribute
        self.children = []
        self.answer = ''
def subtables(data , col , delete):
    dic = {}
    coldata = [row[col] for row in data]
    attr = list(set(coldata))
    for k in attr:
        dic[k] = []
    for y in range(len(data)):
        key = data[y][col]
        if delete:
            del data[y][col]
        dic[key].append(data[y])
    return attr , dic

def entropy(S):
    attr = list(set(S))
    if len(attr) == 1:
        return 0
    counts = [0,0]
    for i in range(2):
        counts[i] = sum( [1 for x in S if attr[i] == x]) / (len(S) * 1.0)
    sums  = 0
    for cnt in counts:
        sums += -1 * cnt * math.log(cnt,2)
    return sums

def compute_gain(data , col):
    attrValues , dic = subtables(data ,col , delete= False)
    total_entropy = entropy([row[-1] for row in data])
    for x in range(len(attrValues)):
        ratio = len(dic[attrValues[x]]) / (len(data) * 1.0 )
        entro = entropy([row[-1] for row in dic[attrValues[x]]])
        total_entropy -= ratio*entro
    return total_entropy

def build_tree(data , features):
    lastcol = [row[-1] for row in data]
    if(len(set(lastcol))) == 1:
        node = Node("")
        node.answer = lastcol[0]
        return node
    n  = len(data[0]) -1
    gains = [compute_gain(data , col) for col in range(n)]
    split = gains.index(max(gains))
    node = Node(features[split])
    fea = features[:split] + features[split+1:]
    attr , dic = subtables(data , split , delete=True)
    for x in range(len(attr)):
        child = build_tree(dic[attr[x]],fea)
        node.children.append((attr[x],child))
    return node
def print_tree(node , level):
    if node.answer != "":
        print(' '*level,node.answer)
        return
    print(' '*(level+1),node.attribute)
    for value , n in node.children:
        print(' '*(level+1),value)
        print_tree(n , level + 2)
def classify(node , x_test , features):
    if node.answer != "":
        print(node.answer)
        return
    pos = features.index(node.attribute)
    for value , n in node.children:
        if x_test[pos] == value:
            classify(n ,x_test,features)
dataset , features = load_csv('data3.csv')
node = build_tree(dataset,features)
print('The decision tree for the dataset using ID3 Algorithm is :')
print_tree(node , 0)
testdata ,features = load_csv('data3_test.csv')
for xtest in testdata:
    print('The test instance :',xtest)
    print('The predicted label : ',end='')
    classify(node,xtest,features)
''')
def run(train_file , test_file):
    dataset, features = load_csv(train_file)
    node = build_tree(dataset, features)
    print('The decision tree for the dataset using ID3 Algorithm is :')
    print_tree(node, 0)
    testdata, features = load_csv(test_file)
    for xtest in testdata:
        print('The test instance :', xtest)
        print('The predicted label : ', end='')
        classify(node, xtest, features)



# if __name__ == '__main__':
#     problem_statement()
#     code()




from aiandml import ao_star,a_star , k_nearest , locally_weighted_regression ,candidate_elimination , id3 , back_propagation ,em_and_k_means, naive_bayesian_classifier
import urllib.request
from PIL import Image
from IPython.display import display
def insertImage(url):
    try:
        urllib.request.urlretrieve(
            url,
            "gfg.jpg")
        img = Image.open("gfg.jpg")
        display(img)
    except Exception:
        print(Exception, 'PLEASE CONNECT TO INTERNET AND TRY TO RERUN')
class aStar:
    def __init__(self , tree, heuristic, start_node, end_node):
        self.tree , self.heuristic , self.start_node , self.end_node =tree , heuristic , start_node , end_node
    def problem_statement(self):
        a_star.problem_statement()
    def description(self):
        a_star.description()
    def algorithm(self):
        a_star.algorithm()
    def code(self):
        a_star.code()
    def run(self):
        print(f'''
Visited Nodes and Optimal Nodes Sequence for tree = {self.tree} 
                                             heuristic = {self.heuristic}
                                             start_node = {self.start_node}
                                             end_node = {self.end_node} ''')
        cost = {self.start_node: 0}
        visited_nodes , optimal_sequence = a_star.astar(self.start_node, self.end_node, self.tree, self.heuristic, cost)
        print(f"Visited Nodes : {visited_nodes} \nOptimal Nodes Sequence : {optimal_sequence}")
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()

class aoStar:
    def __init__(self , graph, heuristicNodeList, startNode):
        self.graph , self.heuristicNodeList , self.startNode = graph, heuristicNodeList, startNode
    def problem_statement(self):
        ao_star.problem_statement()
    def description(self):
        ao_star.description()
    def algorithm(self):
        ao_star.algorithm()
    def code(self):
        ao_star.code()
    def run(self):
        print(f'''
Output of AO* algorithm of
    graph = {self.graph} 
    heuristicNodeList = {self.heuristicNodeList}
    startNode = {self.startNode}
                                                     ''')
        G1 = ao_star.Graph(self.graph, self.heuristicNodeList, self.startNode)
        G1.applyAOstar()
        G1.printSolution()
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class candidateElimination:
    def __init__(self , dataset_path):
        self.dataset_path = dataset_path
    def problem_statement(self):
        candidate_elimination.problem_statement()
    def description(self):
        candidate_elimination.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152000935-3ad58667-584b-4f2e-a3ae-23beecec2d83.png')
    def code(self):
        candidate_elimination.code()
    def run(self):
        candidate_elimination.run(self.dataset_path)
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class id_3:
    def __init__(self , train_dataset_path , test_dataset_path):
        self.train_dataset_path , self.test_dataset_path = train_dataset_path , test_dataset_path
    def problem_statement(self):
        id3.problem_statement()
    def description(self):
        id3.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152000986-c41a480c-f27e-4ec7-805f-c078a8da8b1b.jpg')
    def code(self):
        id3.code()
    def run(self):
        id3.run(self.train_dataset_path , self.test_dataset_path)
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class backPropagation:
    def problem_statement(self):
        back_propagation.problem_statement()
    def description(self):
        back_propagation.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152000889-ec987289-9b9e-478e-bd0f-b30cf7f5088d.jpg')
        insertImage('https://user-images.githubusercontent.com/67971078/152000840-dd7933ea-aab1-4c5e-abb2-6317f6606c4d.png')
    def code(self):
        back_propagation.code()
    def run(self):
        back_propagation.run()
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class naiveBayesianClassifier:
    def __init__(self , dataset_path):
        self.dataset_path = dataset_path
    def problem_statement(self):
        naive_bayesian_classifier.problem_statement()
    def description(self):
        naive_bayesian_classifier.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152001033-d507f8df-470c-4893-a895-19e72d3cd92a.png')
    def code(self):
        naive_bayesian_classifier.code()
    def run(self):
        naive_bayesian_classifier.run(self.dataset_path)
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class emAndKMeans:
    # No external Dataset
    def problem_statement(self):
        em_and_k_means.problem_statement()
    def description(self):
        em_and_k_means.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152000963-dbad1286-14a7-4fec-80b5-b46f4c34e2a1.png')
    def code(self):
        em_and_k_means.code()
    def run(self):
        em_and_k_means.run()
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class kNearestNeighbour:
    # No external Dataset
    def problem_statement(self):
        k_nearest.problem_statement()
    def description(self):
        k_nearest.description()
    def algorithm(self):
        print('''The k-NEAREST NEIGHBOR algorithm is easily adapted to approximating continuous-valued target functions.''')
        insertImage('https://user-images.githubusercontent.com/67971078/152000997-35a963aa-3e56-4e70-a816-96d33b9e019c.png')
    def code(self):
        k_nearest.code()
    def run(self):
        k_nearest.run()
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
class locallyWeightedRegression:
    def __init__(self , dataset_path):
        self.dataset_path = dataset_path
    def problem_statement(self):
        locally_weighted_regression.problem_statement()
    def description(self):
        locally_weighted_regression.description()
    def algorithm(self):
        insertImage('https://user-images.githubusercontent.com/67971078/152001014-dc006daa-6c7a-4fef-a3dd-605e65d4b9d4.png')
    def code(self):
        locally_weighted_regression.code()
    def run(self):
        locally_weighted_regression.run(self.dataset_path)
    def all(self):
        self.problem_statement()
        self.description()
        self.algorithm()
        self.code()
        self.run()
# path = "D:/Documents/Documents/Downloads/ML-CSV/ML-CSV/pima-indians-diabetes.csv"
# naiveBayesianClassifier(path).all()
# naiveBayesianClassifier(path).description()
# path1 = "D:/Documents/Documents/Downloads/ML-CSV/ML-CSV/data3.csv"
# path2 = "D:/Documents/Documents/Downloads/ML-CSV/ML-CSV/data3_test.csv"
# id_3(path1,path2).all()


# path1 = "D:/Documents/Documents/Downloads/ML-CSV/ML-CSV/ws.csv"
# candidateElimination(path1).all()
# tree = {'S': [['A', 1], ['B', 2]],
#         'A': [['E', 13]],
#         'B': [['E', 5]]}
# heuristic = {'S': 5, 'A': 4, 'B': 5, 'E': 0}
# start_node = 'S'
# end_node = 'E'
# aStar(tree,heuristic,start_node,end_node).all()


# h1 = {'A': 1, 'B': 6, 'C': 2, 'D': 12, 'E': 2, 'F': 1, 'G': 5, 'H': 7, 'I': 0, 'J': 1, 'T': 3}
# graph1 = {
#         'A': [[('B', 1), ('C', 1)], [('D', 1)]],
#         'B': [[('G', 1)], [('H', 1)]],
#         'C': [[('J', 1)]],
#         'D': [[('E', 1), ('F', 1)]],
#         'G': [[('I', 1)]]
#     }
# G1 = aoStar(graph1 , h1 , 'A')
# G1.all()

# h2 = {'A': 1, 'B': 6, 'C': 12, 'D': 10, 'E': 4, 'F': 4, 'G': 5, 'H': 7}
# graph2 = {
#         'A': [[('B', 1), ('C', 1)], [('D', 1)]],
#         'B': [[('G', 1)], [('H', 1)]],
#         'D': [[('E', 1), ('F', 1)]]
#     }
# G2 = aoStar(graph2, h2, 'A')
# G2.applyAOstar()
# G2.printSolution()


# kNearestNeighbour().all()
# locallyWeightedRegression().all()


# locallyWeightedRegression('C:/Users/ASHLESH M D/Desktop/Eclipse Data/Tips.csv').run()


from PIL import Image as Image1
from IPython.display import Image as Image2,display
import numpy as np
def sigmoid(x):
    return 1/(1+np.exp(-x))
def sigmoid_grad(x):
    return x*(1-x)
def problem_statement():
    print(
        '''
Problem Statement: 
    Build an Artificial Neural Network by implementing the Back propagation algorithm and test the same using appropriate data sets.     
        '''
    )
def description():
    print(
'Description:\n\tThe BACKPROPAGATION algorithm learns the weights for a multilayer network, given a network with a fixed set of units and interconnections. It employs gradient descent to attempt to minimize the squared error between the network output values and the target values for these outputs. The learning problem faced by BACKPROPAGATION algorithm is to search a large hypothesis space defined by all possible weight values for all the units in the network.'
    )
def algorithm():
    print('The BACKPROPAGATION algorithm is discussed below:')
    try:
        display(Image2('back_propagation2.jpg'))
        display(Image2('back_propagation.png'))
    except:
        Image1.open('back_propagation2.jpg').show()
        Image1.open('back_propagation.png').show()
def code():
    print('\nProgram for Back Propagation Algorithm: ')
    print('''
import numpy as np
X=np.array(([2,9],[1,5],[3,6]),dtype=float)
Y=np.array(([92],[86],[89]),dtype=float)
X=X/np.amax(X,axis=0)
Y=Y/100

def sigmoid(x):
    return 1/(1+np.exp(-x))

def sigmoid_grad(x):
    return x*(1-x)
epoch=1000
eta=0.2
input_neutrons=2
hidden_neutrons=3
output_neutrons=1
wh=np.random.uniform(size=(input_neutrons,hidden_neutrons))
bh=np.random.uniform(size=(1,hidden_neutrons))
wout=np.random.uniform(size=(hidden_neutrons,output_neutrons))
bout=np.random.uniform(size=(1,output_neutrons))

for i in range(epoch):
    h_ip=np.dot(X,wh)+bh
    h_act=sigmoid(h_ip)
    o_ip=np.dot(h_act,wout)
    output=sigmoid(o_ip)
    Eo=Y-output
    outgrad=sigmoid_grad(output)
    d_output=Eo*outgrad
    Eh=d_output.dot(wout.T)
    hiddengrad=sigmoid_grad(h_act)
    d_hidden=Eh*hiddengrad
    wout +=h_act.T.dot(d_output)*eta
    wh +=X.T.dot(d_hidden)*eta
print("Normalized input:\\n"+str(X))
print("Actual output:\\n"+str(Y))
print("Predicted  output:\\n",output)
    ''')
def run():
    X = np.array(([2, 9], [1, 5], [3, 6]), dtype=float)
    Y = np.array(([92], [86], [89]), dtype=float)
    X = X / np.amax(X, axis=0)
    Y = Y / 100
    epoch = 1000
    eta = 0.2
    input_neutrons = 2
    hidden_neutrons = 3
    output_neutrons = 1
    wh = np.random.uniform(size=(input_neutrons, hidden_neutrons))
    bh = np.random.uniform(size=(1, hidden_neutrons))
    wout = np.random.uniform(size=(hidden_neutrons, output_neutrons))
    bout = np.random.uniform(size=(1, output_neutrons))

    for i in range(epoch):
        h_ip = np.dot(X, wh) + bh
        h_act = sigmoid(h_ip)
        o_ip = np.dot(h_act, wout)
        output = sigmoid(o_ip)
        Eo = Y - output
        outgrad = sigmoid_grad(output)
        d_output = Eo * outgrad
        Eh = d_output.dot(wout.T)
        hiddengrad = sigmoid_grad(h_act)
        d_hidden = Eh * hiddengrad
        wout += h_act.T.dot(d_output) * eta
        wh += X.T.dot(d_hidden) * eta
    print("Normalized input:\n" + str(X))
    print("Actual output:\n" + str(Y))
    print("Predicted  output:\n", output)

# if __name__ == '__main__':
#     run()


from PIL import Image as Image1
from IPython.display import Image as Image2,display
def problem_statement():
    print('''
Problem Statement: 
    Write a program to implement the naïve Bayesian classifier for a sample training data set stored 
    as a .CSV file. Compute the accuracy of the classifier, considering few test data sets.''')
def description():
    print('Description:\n\tNaive Bayes classification algorithm can be extremely fast relative to other classification algorithms. It works on Bayes theorem of probability to predict the class of unknown data set. In simple terms, a Naive Bayes classifier assumes that the presence of a particular feature in a class is unrelated to the presence of any other feature. For example, a fruit may be considered to be an apple if it is red, round, and about 3 inches in diameter. Even if these features depend on each other or upon the existence of the other features, all of these properties independently contribute to the probability that this fruit is an apple and that is  why it is known as ‘Naive’. Naive Bayes model is easy to build and particularly useful for very large data sets.')
def algorithm():
    print('Algorithm:')
    try:
        display(Image2('naive_bayesian_classifier.png'))
    except:
        Image1.open('naive_bayesian_classifier.png').show()
def code():
    print('\nProgram for Naive Bayesian Classifier :')
    print('''
import statistics as st, csv ,random ,math , stat
def loadcsv(filename):
    lines = csv.reader(open(filename, "r"))
    dataset = list(lines)
    for i in range(len(dataset)):
        dataset[i] = [float(x) for x in dataset[i]]
    return dataset
def splitDataset(dataset, splitRatio):
    trainSize = int(len(dataset) * splitRatio)
    trainSet = []
    copy = list(dataset)
    while len(trainSet) < trainSize:
        index = random.randrange(len(copy)) # random index
        trainSet.append(copy.pop(index))
    return [trainSet, copy]
def separateByClass(dataset):
    separated = {}
    for i in range(len(dataset)):
        vector = dataset[i]
        if (vector[-1] not in separated):
            separated[vector[-1]] = []
        separated[vector[-1]].append(vector)
    return separated
def summarize(dataset):
    summaries = [(st.mean(attribute), st.stdev(attribute)) for attribute in zip(*dataset)]
    del summaries[-1]
    return summaries
def summarizeByClass(dataset):
    separated = separateByClass(dataset)
    summaries = {}
    for classValue, instances in separated.items():
        summaries[classValue] = summarize(instances)
    return summaries
def calculateProbability(x, mean, stdev):
    exponent = math.exp(-(math.pow(x-mean,2)/(2*math.pow(stdev,2))))
    return (1 / (math.sqrt(2*math.pi) * stdev)) * exponent
#3.2 Calculate Class Probabilities
def calculateClassProbabilities(summaries, inputVector):
    probabilities = {}
    for classValue, classSummaries in summaries.items():
        probabilities[classValue] = 1
        for i in range(len(classSummaries)):
            mean, stdev = classSummaries[i]
            x = inputVector[i]
            probabilities[classValue] *= calculateProbability(x, mean, stdev)
    return probabilities
def predict(summaries, inputVector):
    probabilities = calculateClassProbabilities(summaries, inputVector)
    bestLabel, bestProb = None, -1
    for classValue, probability in probabilities.items():
        if bestLabel is None or probability > bestProb:
            bestProb = probability
            bestLabel = classValue
    return bestLabel
def getPredictions(summaries, testSet):
    predictions = []
    for i in range(len(testSet)):
        result = predict(summaries, testSet[i])
        predictions.append(result)
    return predictions
#5. Computing Accuracy
def getAccuracy(testSet, predictions):
    correct = 0
    for i in range(len(testSet)):
        if testSet[i][-1] == predictions[i]:
            correct += 1
    return (correct/float(len(testSet))) * 100.0
if __name__ == '__main__':
    filename = 'pima-indians-diabetes.csv'
    splitRatio = 0.67
    dataset = loadcsv(filename)
    print("The length of the Data Set : ",len(dataset))

    print("The Data Set Splitting into Training and Testing \\n")
    trainingSet, testSet = splitDataset(dataset, splitRatio)

    print('Number of Rows in Training Set:{0} rows'.format(len(trainingSet)))
    print('Number of Rows in Testing Set:{0} rows'.format(len(testSet)))

    print("\\nFirst Five Rows of Training Set:")
    for i in range(0,5):
        print(trainingSet[i],"\\n")
    print("\\nFirst Five Rows of Testing Set:")
    for i in range(0,5):
        print(testSet[i],"\\n")
 # prepare model
    summaries = summarizeByClass(trainingSet)
    predictions = getPredictions(summaries, testSet)
    accuracy = getAccuracy(testSet, predictions)
    print('\\n Accuracy: {0}%'.format(accuracy))
''')
import statistics as st, csv ,random ,math , stat
def loadcsv(filename):
    lines = csv.reader(open(filename, "r"))
    dataset = list(lines)
    for i in range(len(dataset)):
        dataset[i] = [float(x) for x in dataset[i]]
    return dataset
def splitDataset(dataset, splitRatio):
    trainSize = int(len(dataset) * splitRatio)
    trainSet = []
    copy = list(dataset)
    while len(trainSet) < trainSize:
        index = random.randrange(len(copy)) # random index
        trainSet.append(copy.pop(index))
    return [trainSet, copy]
def separateByClass(dataset):
    separated = {}
    for i in range(len(dataset)):
        vector = dataset[i]
        if (vector[-1] not in separated):
            separated[vector[-1]] = []
        separated[vector[-1]].append(vector)
    return separated
def summarize(dataset):
    summaries = [(st.mean(attribute), st.stdev(attribute)) for attribute in zip(*dataset)]
    del summaries[-1]
    return summaries
def summarizeByClass(dataset):
    separated = separateByClass(dataset)
    summaries = {}
    for classValue, instances in separated.items():
        summaries[classValue] = summarize(instances)
    return summaries
def calculateProbability(x, mean, stdev):
    exponent = math.exp(-(math.pow(x-mean,2)/(2*math.pow(stdev,2))))
    return (1 / (math.sqrt(2*math.pi) * stdev)) * exponent
#3.2 Calculate Class Probabilities
def calculateClassProbabilities(summaries, inputVector):
    probabilities = {}
    for classValue, classSummaries in summaries.items():
        probabilities[classValue] = 1
        for i in range(len(classSummaries)):
            mean, stdev = classSummaries[i]
            x = inputVector[i]
            probabilities[classValue] *= calculateProbability(x, mean, stdev)
    return probabilities
def predict(summaries, inputVector):
    probabilities = calculateClassProbabilities(summaries, inputVector)
    bestLabel, bestProb = None, -1
    for classValue, probability in probabilities.items():
        if bestLabel is None or probability > bestProb:
            bestProb = probability
            bestLabel = classValue
    return bestLabel
def getPredictions(summaries, testSet):
    predictions = []
    for i in range(len(testSet)):
        result = predict(summaries, testSet[i])
        predictions.append(result)
    return predictions
def getAccuracy(testSet, predictions):
    correct = 0
    for i in range(len(testSet)):
        if testSet[i][-1] == predictions[i]:
            correct += 1
    return (correct/float(len(testSet))) * 100.0
def run(filename):
    splitRatio = 0.67
    dataset = loadcsv(filename)
    print("The length of the Data Set : ",len(dataset))

    print("The Data Set Splitting into Training and Testing \n")
    trainingSet, testSet = splitDataset(dataset, splitRatio)

    print('Number of Rows in Training Set:{0} rows'.format(len(trainingSet)))
    print('Number of Rows in Testing Set:{0} rows'.format(len(testSet)))

    print("\nFirst Five Rows of Training Set:")
    for i in range(0,5):
        print(trainingSet[i],"\n")
    print("\nFirst Five Rows of Testing Set:")
    for i in range(0,5):
        print(testSet[i],"\n")
    summaries = summarizeByClass(trainingSet)
    predictions = getPredictions(summaries, testSet)
    accuracy = getAccuracy(testSet, predictions)
    print('\nAccuracy: {0}%'.format(accuracy))
# run()


from PIL import Image as Image1
from IPython.display import Image as Image2,display
import numpy as np
import pandas as pd

def learn(concept, target):
    specific_h = concepts[0].copy()
    generic_h = [['?' for i in range(len(specific_h))] for i in range(len(specific_h))]
    print(
        'Initialing specific_h and generic_h'
        '\nSpecific Boundaries :', specific_h,
        '\nGeneric Boundaries : ', generic_h
    )
    for i, h in enumerate(concept):
        print('\nInstance ',i+1,' is ',h)
        if target[i] == 'Yes':
            print('Instance is Positive')
            for x in range(len(specific_h)):
                if h[x] != specific_h[x]:
                    specific_h[x], generic_h[x][x] = '?', '?'
        if target[i] == 'No':
            print('Instance is Negative')
            for x in range(len(specific_h)):
                generic_h[x][x] = specific_h[x] if h[x] != specific_h[x] else '?'
        print(
            'Specific Boundaries after ', i + 1, ' instances is ', specific_h,
            '\nGeneric Boundaries after ', i + 1, ' instances is ', generic_h
        )
    indices = ['?' for i, val in enumerate(generic_h) if val == ['?'] * 6]
    for i in indices:
        generic_h.remove(['?'] * 6)
    print(
        '\nFinal Specific_h : ', specific_h,
        '\nFinal Generic_h : ', generic_h
    )

def problem_statement():
    print('''
Problem Statement:
    For a given set of training data examples stored in a .CSV file, implement and demonstrate the Candidate-Elimination algorithm to output a description of the set of all hypotheses consistent with the training examples.
    ''')
def description():
    print('Description:\n\tCandidate-Elimination algorithm is used to find all the set of hypothesis consistent with the given data sample. It uses version spaces & considers both positive & negative results. Consider the dataset same as that of the Find-S algorithm for implementation.')
# def algorithm():
#     try:
#         display(Image2('candidate_elimination_algo.png'))
#     except:
#         Image1.open('candidate_elimination_algo.png').show()
def code():
    print('\nProgram for Candidate-Elimination algorithm')
    print(
        '''
import numpy as np
import pandas as pd
data = pd.read_csv('ws.csv', header=None)
concepts, target = np.array(data.iloc[:, 0:-1]), np.array(data.iloc[:, -1])
def learn(concept, target):
    specific_h = concepts[0].copy()
    generic_h = [['?' for i in range(len(specific_h))] for i in range(len(specific_h))]
    print(
        'Initialing specific_h and generic_h'
        '\\nSpecific Boundaries :', specific_h,
        '\\nGeneric Boundaries : ', generic_h
    )
    for i, h in enumerate(concept):
        print('\\nInstance ',i+1,' is ',h)
        if target[i] == 'Yes':
            print('Instance is Positive')
            for x in range(len(specific_h)):
                if h[x] != specific_h[x]:
                    specific_h[x], generic_h[x][x] = '?', '?'
        if target[i] == 'No':
            print('Instance is Negative')
            for x in range(len(specific_h)):
                generic_h[x][x] = specific_h[x] if h[x] != specific_h[x] else '?'
        print(
            'Specific Boundaries after ', i + 1, ' instances is ', specific_h,
            '\\nGeneric Boundaries after ', i + 1, ' instances is ', generic_h
        )
    indices = ['?' for i, val in enumerate(generic_h) if val == ['?'] * 6]
    for i in indices:
        generic_h.remove(['?'] * 6)
    print(
        '\\nFinal Specific_h : ', specific_h,
        '\\nFinal Generic_h : ', generic_h
    )
if __name__ == '__main__':
    learn(concepts, target)
        '''
    )
def run(filename):
    data = pd.read_csv(filename, header=None)
    global concepts
    concepts, target = np.array(data.iloc[:, 0:-1]), np.array(data.iloc[:, -1])
    learn(concepts, target)


from setuptools import setup, find_packages

setup(
    name="aiandml",
    version="1.0.6",
    description=""" The description of the package   """,
    long_description_content_type="text/markdown",
    long_description=open('README.txt').read(),
    # url="https://github.com/ashleshmd/aiandml",
    author="Ashlesh M D",
    author_email='ashleshmd@gmail.com',
    classifiers=["License :: OSI Approved :: MIT License",
                 "Programming Language :: Python :: 3",
                 "Programming Language :: Python :: 3.8",
                 "License :: OSI Approved :: MIT License",
                 "Operating System :: OS Independent",
                 ],
    packages=find_packages(),
    python_requires=">=3.6",
    include_package_data=True,
)


from PIL import Image as Image1
from IPython.display import Image as Image2,display
def problem_statement():
    print('''
Problem Statement:
    Write a program to implement k-Nearest Neighbour algorithm to classify the iris data set. Print both correct and wrong predictions. Java/Python ML library classes can be used for this problem.
    ''')
def description():
    print('Description:\n\tThe most basic instance-based method is the k-NEAREST NEIGHBOR algorithm. This algorithm assumes all instances correspond to points in the n- dimensional space Rn. The nearest neighbors of an instance are defined in terms of the standard Euclidean distance. In nearest-neighbor learning the target function may be either discrete-valued or real-valued.Let us first consider learning discrete-valued target functions of the form f: Rn→V, where V is the finite set {vl, . . . vs}.')
def algorithm():
    print('''The k-NEAREST NEIGHBOR algorithm is easily adapted to approximating continuous-valued target functions.''')
    try:
        display(Image2('k_nearest.png'))
    except:
        Image1.open('k_nearest.png').show()
def code():
    print('\nProgram for K-Nearest Neighbors Algorithm: ')
    print('''
from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
from sklearn import datasets
iris = datasets.load_iris()
print('Iris Data Set Loaded...')
x_train, x_test, y_train, y_test = train_test_split(iris.data, iris.target, test_size= 0.1)
print(
        'Dataset is split into training and testing...'
        '\\nSize of training data and its label ', x_train.shape, y_train.shape,
        '\\nSize of testing data and its label ', x_test.shape, y_test.shape
    )
for i in range(len(iris.target_names)):
    print('Label', i, '-', iris.target_names[i])

classifier = KNeighborsClassifier(n_neighbors=1)
classifier.fit(x_train, y_train)
y_prediction = classifier.predict(x_test)
print('Result of Classification using K-n with K=1')
for r in range(len(x_test)):
    print('Sample: ', x_test[r],
              'Actual-label: ', y_test[r],
              'Predicted-label: ', y_prediction[r]
              )
print('Classification Accuracy : ', classifier.score(x_test, y_test))
''')
def run():
    from sklearn.model_selection import train_test_split
    from sklearn.neighbors import KNeighborsClassifier
    from sklearn import datasets
    iris = datasets.load_iris()
    iris = datasets.load_iris()
    print('Iris Data Set Loaded...')
    x_train, x_test, y_train, y_test = train_test_split(iris.data, iris.target, test_size=0.1)
    print(
        'Dataset is split into training and testing...'
        '\nSize of training data and its label ', x_train.shape, y_train.shape,
        '\nSize of testing data and its label ', x_test.shape, y_test.shape
    )
    for i in range(len(iris.target_names)):
        print('Label', i, '-', iris.target_names[i])

    classifier = KNeighborsClassifier(n_neighbors=1)
    classifier.fit(x_train, y_train)
    y_prediction = classifier.predict(x_test)
    print('Result of Classification using K-n with K=1')
    for r in range(len(x_test)):
        print('Sample: ', x_test[r],
              'Actual-label: ', y_test[r],
              'Predicted-label: ', y_prediction[r]
              )
    print('Classification Accuracy : ', classifier.score(x_test, y_test))

# if __name__ == '__main__':
#     run()